@@ -34,7 +34,7 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
