@@ -515,8 +515,13 @@ svc_vc_rendezvous(SVCXPRT *xprt)
 		SVC_DESTROY(newxprt);
 		/* Was never added to epoll */
 		SVC_RELEASE(newxprt, SVC_RELEASE_FLAG_NONE);
+		SVC_RELEASE(xprt, SVC_RELEASE_FLAG_NONE);
 		return (XPRT_DESTROYED);
 	}
+
+	/* We're not using a ref for the hook anymore, since epoll doesn't store
+	 * the transport pointer.  Drop the extra ref here. */
+	SVC_RELEASE(newxprt, SVC_RELEASE_FLAG_NONE);
 	return (XPRT_IDLE);
 }
 
@@ -562,6 +567,12 @@ svc_vc_destroy_task(struct work_pool_entry *wpe)
 	svc_vc_xprt_free(VC_DR(rec));
 }
 
+static void
+svc_vc_unlink_it(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
+{
+	svc_rqst_xprt_unregister(xprt, flags);
+}
+
 static void
 svc_vc_destroy_it(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
 {
@@ -570,8 +581,6 @@ svc_vc_destroy_it(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
 		.tv_nsec = 0,
 	};
 
-	svc_rqst_xprt_unregister(xprt, flags);
-
 	__warnx(TIRPC_DEBUG_FLAG_REFCNT,
 		"%s() %p fd %d xp_refcnt %" PRId32 " @%s:%d",
 		__func__, xprt, xprt->xp_fd, xprt->xp_refcnt, tag, line);
@@ -586,12 +595,6 @@ svc_vc_destroy_it(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
 	work_pool_submit(&svc_work_pool, &(REC_XPRT(xprt)->ioq.ioq_wpe));
 }
 
-static void
-svc_vc_destroy(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
-{
-	svc_vc_destroy_it(xprt, flags, tag, line);
-}
-
 extern mutex_t ops_lock;
 
  /*ARGSUSED*/
@@ -981,7 +984,8 @@ svc_vc_override_ops(SVCXPRT *xprt, SVCXPRT *rendezvous)
 		ops.xp_decode = svc_vc_decode;
 		ops.xp_reply = svc_vc_reply;
 		ops.xp_checksum = svc_vc_checksum;
-		ops.xp_destroy = svc_vc_destroy;
+		ops.xp_unlink = svc_vc_unlink_it;
+		ops.xp_destroy = svc_vc_destroy_it;
 		ops.xp_control = svc_vc_control;
 		ops.xp_free_user_data = NULL;	/* no default */
 	}
@@ -1006,6 +1010,7 @@ svc_vc_rendezvous_ops(SVCXPRT *xprt)
 		ops.xp_decode = (svc_req_fun_t)abort;
 		ops.xp_reply = (svc_req_fun_t)abort;
 		ops.xp_checksum = NULL;		/* not used */
+		ops.xp_unlink = svc_vc_unlink_it;
 		ops.xp_destroy = svc_vc_destroy_it;
 		ops.xp_control = svc_vc_rendezvous_control;
 		ops.xp_free_user_data = NULL;	/* no default */
